package handlers

import (
	"database/sql"
	"fmt"
	"net/http"

	"git.duckylabs.xyz/duckbox/echo-todo/models"
	"git.duckylabs.xyz/duckbox/echo-todo/partials"
	"github.com/labstack/echo/v4"
)

func AddUserTableHandler(c echo.Context) error {
	return render(c, partials.AddUserTable())
}

func AllUserTableHandler(c echo.Context) error {
	logger := c.Logger()
	db := c.Get("db").(*sql.DB)

	rows, err := db.Query("SELECT * FROM users")
	if err != nil {
		logger.Fatalf("failed to execute query: %v\n", err)
		return c.String(http.StatusInternalServerError, "<h4>There was an error fetching users</h4>")
	}

	defer rows.Close()

	var users []models.User

	for rows.Next() {
		var user models.User

		if err := rows.Scan(&user.ID, &user.Username, &user.Email, &user.Role); err != nil {
			logger.Fatalf("Error scanning row: %v", err)
			return c.String(http.StatusInternalServerError, "<h4>There was an error fetching users</h4>")
		}

		users = append(users, user)
		logger.Printf("ID: %d, Username: %s, Email: %s, Role: %s\n", user.ID, user.Username, user.Email, user.Role)
	}

	if err := rows.Err(); err != nil {
		logger.Fatalf("Error during rows iteration: %v", err)
	}

	return render(c, partials.UserTable(users))
}

func AddUserHandler(c echo.Context) error {
	username := c.FormValue("username")
	email := c.FormValue("email")
	role := c.FormValue("role")

	// Perform validation on the form values
	if username == "" {
		return c.String(http.StatusBadRequest, "<h4>Username is required</h4>")
	}
	if email == "" {
		return c.String(http.StatusBadRequest, "<h4>Email is required</h4>")
	}
	if role == "" {
		return c.String(http.StatusBadRequest, "<h4>Role is required</h4>")
	}

	user := models.User{
		Username: username,
		Email:    email,
		Role:     role,
	}

	// TODO: Insert user into DB if valid

	fmt.Printf("%v", user)
	// return c.JSON(http.StatusCreated, u)
	return c.String(http.StatusOK, "<h4>Successfully added "+username+"</h4>")
}
